import { AxisBox2D, Point2D } from "../../../types/geometry";
import { ResolvedValues, VisualElement } from "../../../render/types";
import { VisualState } from "../../../render/utils/state";
import { Transition } from "../../../types";
export interface Snapshot {
    isDragging?: boolean;
    cursorProgress?: Point2D;
    latestMotionValues: ResolvedValues;
    boundingBox?: AxisBox2D;
}
export declare type LeadAndFollow = [
    VisualElement | undefined,
    VisualElement | undefined
];
/**
 * For each layout animation, we want to identify two components
 * within a stack that will serve as the "lead" and "follow" components.
 *
 * In the switch animation, the lead component performs the entire animation.
 * It uses the follow bounding box to animate out from and back to. The follow
 * component is hidden.
 *
 * In the crossfade animation, both the lead and follow components perform
 * the entire animation, animating from the follow origin bounding box to the lead
 * target bounding box.
 *
 * Generalising a stack as First In Last Out, *searching from the end* we can
 * generally consider the lead component to be:
 *  - If the last child is present, the last child
 *  - If the last child is exiting, the last *encountered* exiting component
 */
export interface LayoutStack {
    add(element: VisualElement): void;
    remove(element: VisualElement): void;
    getLead(): VisualElement | undefined;
    updateSnapshot(): void;
    clearSnapshot(): void;
    animate(element: VisualElement, crossfade: boolean): void;
    updateLeadAndFollow(): void;
}
export interface CrossfadeState {
    isCrossfading(): boolean;
    getValues(element: VisualElement): VisualState;
    crossfadeFromLead(transition?: Transition): void;
    crossfadeToLead(transition?: Transition): void;
    reset(): void;
}
export declare function createCrossfadeState(state: StackState, forceCrossfade?: boolean): CrossfadeState;
interface StackState {
    lead?: VisualElement;
    follow?: VisualElement;
    leadIsExiting: boolean;
}
export declare function layoutStack(): LayoutStack;
export {};
